import time

import allure
import pytest

from refinitiv.data import OpenState
from refinitiv.data._content_type import ContentType
from refinitiv.data._errors import ScopeError
from refinitiv.data.content import pricing
from refinitiv.data.delivery._stream import stream_cxn_cache
from refinitiv.data.delivery.omm_stream import ContribType
from refinitiv.data.errors import RDError
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.content.helpers import (
    add_callbacks_for_universe_stream,
    check_triggered_events,
    check_stream_data,
)
from tests.integration.content.pricing.conftest import (
    check_pricing_data,
    check_item_message_and_status,
    add_callback_for_contribution,
)
from tests.integration.delivery.stream.omm_stream.conftest import on_ack, on_error
from tests.integration.helpers import (
    get_async_response_from_definitions,
    check_contrib_response,
    check_response_status,
)

HALF_SECOND = 0.5


@allure.suite("Content object - Pricing")
@allure.feature("Content object - Pricing")
@allure.severity(allure.severity_level.CRITICAL)
class TestPricing:
    @allure.title(
        "Open a Pricing stream on session and call universe without fields params"
    )
    @pytest.mark.parametrize(
        "universe,fields", [("EUR=", "IRGPRC"), ("GBP=", ["BID", "ASK"])]
    )
    @pytest.mark.caseid("35070172")
    def test_open_pricing_stream_by_universe_without_fields(
        self, universe, fields, open_session
    ):
        pricing_stream = pricing.Definition(
            universe=universe, api="streaming.pricing.endpoints.main"
        ).get_stream(session=open_session)

        check_stream_data(pricing_stream, universe, fields)

    @allure.title(
        "Creating Pricing definition object specifying the universe, fields and get data"
    )
    @pytest.mark.parametrize(
        "universe,fields", [("EUR=", "IRGPRC"), (["USD=", "GBP="], ["BID", "ASK"])]
    )
    @pytest.mark.caseid("35158864")
    def test_create_pricing_object_by_universe_with_fields(
        self, universe, fields, open_platform_session
    ):
        pricing_definition = pricing.Definition(
            universe=universe, fields=fields
        ).get_data(session=open_platform_session)

        check_pricing_data(pricing_definition, universe, fields)

    @allure.title("Open a Pricing stream on session and check item status")
    @pytest.mark.parametrize("universe", ["EUR="])
    @pytest.mark.caseid("35167789")
    def test_open_pricing_stream_by_universe_and_check_item_status(
        self, universe, open_session
    ):
        pricing_stream = pricing.Definition(universe=universe).get_stream(
            session=open_session
        )
        item_status = pricing_stream.open()

        assert (
            item_status == OpenState.Opened
        ), f"Item status is not Open, actual status is {item_status}"
        pricing_stream.close()

    @allure.title("Create a Pricing stream connection and check events")
    @pytest.mark.parametrize(
        "universe,expected_events",
        [(["EUR=", "GBP="], ["Refresh", "Refresh", "Complete", "Update"])],
    )
    @pytest.mark.caseid("35168704")
    @pytest.mark.smoke
    def test_open_pricing_stream_and_check_events(
        self, universe, expected_events, open_desktop_session
    ):
        triggered_events = []
        pricing_stream = pricing.Definition(universe=universe).get_stream()
        add_callbacks_for_universe_stream(pricing_stream, triggered_events)

        pricing_stream.open(with_updates=True)
        time.sleep(3)
        pricing_stream.close()

        check_triggered_events(triggered_events, expected_events)
        assert (
            triggered_events.count(expected_events[2]) == 1
        ), f"Callback on_complete called more than once: {triggered_events.count(expected_events[1])}"

    @allure.title("Create a Pricing stream with an invalid universe name")
    @pytest.mark.parametrize(
        "universe,expected_message",
        [
            ("INVALID", "The record could not be found"),
            (["EUR=", "INVAL"], "The record could not be found"),
        ],
    )
    @pytest.mark.caseid("35360349")
    def test_create_pricing_stream_with_invalid_data(
        self, universe, expected_message, open_desktop_session
    ):
        pricing_stream = pricing.Definition(universe=universe).get_stream(
            session=open_desktop_session
        )
        pricing_stream.open()

        check_item_message_and_status(universe, expected_message, pricing_stream)

    @allure.title(
        "Create a Pricing stream with user creds which does not have permission"
    )
    @pytest.mark.parametrize(
        "universe,fields,expected_message",
        [
            ("USD=", "BID", "A21: DACS User Profile denied access to vendor\n"),
            (
                ["EUR=", "USD="],
                ["BID", "ASK"],
                "A21: DACS User Profile denied access to vendor\n",
            ),
        ],
    )
    @pytest.mark.caseid("C43420983")
    def test_create_pricing_stream_with_user_creds_which_does_not_have_permission(
        self, universe, fields, expected_message, open_platform_session_with_rdp_creds
    ):
        pricing_stream = pricing.Definition(universe=universe).get_stream(
            session=open_platform_session_with_rdp_creds
        )
        pricing_stream.open()

        check_item_message_and_status(universe, expected_message, pricing_stream)

    @allure.title("Create a Pricing stream connection without updates and check events")
    @pytest.mark.parametrize("universe,fields", [("EUR=", "BID")])
    @pytest.mark.caseid("35364411")
    def test_open_pricing_stream_without_update(self, universe, open_session, fields):
        triggered_events_list = []
        pricing_stream = pricing.Definition(
            universe=universe, fields=fields
        ).get_stream()
        add_callbacks_for_universe_stream(pricing_stream, triggered_events_list)

        pricing_stream.open(with_updates=False)
        pricing_stream.close()

        assert "Complete" in triggered_events_list, f"{triggered_events_list}"
        assert (
            "Update" not in triggered_events_list
        ), f"Update is exist, {triggered_events_list}"

    @allure.title(
        "Creation of two pricing streams in one session and receiving data on them"
    )
    @pytest.mark.parametrize("universe_01,universe_02", [("EUR=", "USD=")])
    @pytest.mark.caseid("35371558")
    def test_open_two_pricing_stream_in_one_session(
        self, universe_01, universe_02, open_session
    ):
        pricing_stream_01 = pricing.Definition(universe=universe_01).get_stream()
        pricing_stream_02 = pricing.Definition(universe=universe_02).get_stream()

        check_stream_data(pricing_stream_01, universe_01)
        check_stream_data(pricing_stream_02, universe_02)

    @allure.title("Create a new stream when session is closed")
    @pytest.mark.parametrize("universe,fields", [("EUR=", "BID")])
    @pytest.mark.caseid("35385930")
    def test_open_pricing_stream_when_connection_was_dropped(
        self, universe, fields, open_session
    ):
        session = open_session
        session.close()
        pricing_stream = pricing.Definition(
            universe=universe, fields=fields
        ).get_stream()
        with pytest.raises(AssertionError) as error:
            pricing_stream.open(with_updates=True)
        assert str(error.value) == "Session must be open"

    @allure.title(
        "Creating Pricing definition object specifying the universe, fields and get_data_async"
    )
    @pytest.mark.parametrize(
        "universe,fields,invalid_universe",
        [("EUR=", "BID", []), (["USD=", "GBP="], ["BID", "ASK"], [])],
    )
    @pytest.mark.caseid("35395377")
    async def test_get_data_async(
        self, universe, fields, invalid_universe, open_platform_session_async
    ):
        valid_response, invalid_response = await get_async_response_from_definitions(
            pricing.Definition(universe=universe, fields=fields),
            pricing.Definition(universe=invalid_universe, fields=fields),
        )

        check_pricing_data(valid_response, universe, fields)
        check_response_status(
            response=invalid_response,
            expected_status_code=HttpStatusCode.FOUR_HUNDRED,
            expected_http_reason=HttpReason.BAD_REQUEST,
            expected_error_code=HttpStatusCode.FOUR_HUNDRED,
            expected_error_message="Validation error: Missing required parameter 'universe'",
        )

    @allure.title("Create Pricing definition object with extended params")
    @pytest.mark.parametrize(
        "universe,fields,extended_universe,extended_fields",
        [("EUR=", "BID", "GBP", "IRGPRC")],
    )
    @pytest.mark.caseid("35395377")
    def test_get_data_by_extended_params(
        self,
        universe,
        fields,
        extended_universe,
        extended_fields,
        open_platform_session,
    ):
        response = pricing.Definition(
            universe=universe,
            fields=fields,
            extended_params={"universe": extended_universe, "fields": extended_fields},
        ).get_data(session=open_platform_session)

        check_pricing_data(response, extended_universe, extended_fields)

    @allure.title(
        "Create Pricing stream with universe and put to extended_params new fields and check fields is exist"
    )
    @pytest.mark.parametrize("universe,extended_fields", [("EUR=", ["IRGPRC"])])
    @pytest.mark.caseid("35401653")
    def test_create_stream_with_universe_and_put_to_extended_params_new_fields(
        self, universe, extended_fields, open_session
    ):
        stream = pricing.Definition(
            universe=universe, extended_params={"View": extended_fields}
        ).get_stream()
        check_stream_data(
            stream, expected_universe=universe, expected_fields=extended_fields
        )

    @allure.title("Closing streaming when calling session.close()")
    @pytest.mark.parametrize("universe,fields", [("EUR=", "BID")])
    @pytest.mark.caseid("35801379")
    def test_closing_streaming_when_calling_session_close(
        self, universe, fields, open_session
    ):
        session = open_session
        pricing_stream = pricing.Definition(
            universe=universe, fields=fields
        ).get_stream()
        pricing_stream.open(with_updates=True)
        session.close()

        is_cnx_alive = stream_cxn_cache.is_cxn_alive(
            session, ContentType.STREAMING_PRICING
        )

        assert not is_cnx_alive, f"Connection is alive"
        assert not pricing_stream._stream.is_opened, f"Stream is not closed"

    @allure.title("Create pricing definition object with invalid params and get error")
    @pytest.mark.caseid("40147423")
    @pytest.mark.parametrize(
        "universe,fields,expected_error",
        [
            (
                [],
                ["", "INVALID"],
                "Error code 400 | Validation error: Missing required parameter 'universe'",
            ),
        ],
    )
    def test_pricing_definition_object_with_invalid_params_and_get_error(
        self,
        universe,
        fields,
        expected_error,
        open_platform_session,
    ):
        with pytest.raises(RDError) as error:
            pricing.Definition(universe=universe, fields=fields).get_data()
        assert str(error.value) == expected_error

    @allure.title("Open and close streaming with context manager")
    @pytest.mark.parametrize("universe,fields", [("EUR=", "BID")])
    @pytest.mark.caseid("C44023013")
    def test_open_streaming_with_context_manager(self, universe, fields, open_session):
        pricing_stream = pricing.Definition(universe, fields=fields).get_stream()
        with pricing_stream:
            assert pricing_stream._stream.is_opened, f"Stream is not Opened"
        assert not pricing_stream._stream.is_opened, f"Stream is not Closed"

    @allure.title("Open and close streaming with async context manager")
    @pytest.mark.parametrize("universe,fields", [("EUR=", "BID")])
    @pytest.mark.caseid("C44023014")
    async def test_open_streaming_with_async_context_manager(
        self, universe, fields, open_session_async
    ):
        pricing_stream = pricing.Definition(universe, fields=fields).get_stream()
        async with pricing_stream:
            assert pricing_stream._stream.is_opened, f"Stream is not Opened"
        assert not pricing_stream._stream.is_opened, f"Stream is not Closed"

    @pytest.mark.caseid("")
    @allure.title("Create pricing stream with invalid api parameter")
    @pytest.mark.parametrize("universe,fields", [("GBP=", "BID")])
    def test_create_pricing_stream_with_invalid_api(
        self, open_session, universe, fields
    ):
        with pytest.raises(ValueError) as error:
            pricing_stream = pricing.Definition(
                universe, api="streaming.pricing.endpoints.invalid", fields=fields
            ).get_stream()
            pricing_stream.open()
        assert (
            str(error.value)
            == "Not an existing path apis.streaming.pricing.endpoints.invalid to url into config file"
        )

    @pytest.mark.caseid("C47579675")
    @allure.title("Add fields to existing stream")
    @pytest.mark.parametrize(
        "universe, fields, new_fields, expected_fields",
        [
            ("GBP=", "BID", ["ASK", "IRGPRC"], ["BID", "ASK", "IRGPRC"]),
            ("EUR=", "BID", "ASK", ["BID", "ASK"]),
        ],
    )
    def test_add_fields_to_existing_pricing_stream(
        self, open_session, universe, fields, new_fields, expected_fields
    ):

        pricing_stream = pricing.Definition(
            universe=universe, fields=fields
        ).get_stream()
        pricing_stream.open()
        time.sleep(HALF_SECOND)
        pricing_stream.add_fields(new_fields)
        time.sleep(HALF_SECOND)
        snapshot = pricing_stream.get_snapshot()
        check_pricing_data(snapshot, universe, expected_fields)

    @pytest.mark.caseid("C47579665")
    @allure.title("Remove fields in existing stream")
    @pytest.mark.parametrize(
        "universe, fields, removed_fields, expected_fields",
        [
            (["GBP=", "EUR="], ["ASK", "IRGPRC"], "ASK", ["IRGPRC"]),
            ("EUR=", "BID", "BID", None),
            ("EUR=", "BID", "ASK", ["BID"]),
        ],
    )
    def test_remove_fields_in_existing_pricing_stream(
        self, open_session, universe, fields, removed_fields, expected_fields
    ):
        pricing_stream = pricing.Definition(
            universe=universe, fields=fields
        ).get_stream()
        pricing_stream.open()
        time.sleep(HALF_SECOND)
        pricing_stream.remove_fields(removed_fields)
        time.sleep(HALF_SECOND)
        snapshot = pricing_stream.get_snapshot()
        check_pricing_data(snapshot, universe, expected_fields)

    @pytest.mark.caseid("C47579526")
    @allure.title("Add and remove fields to existing stream")
    @pytest.mark.parametrize(
        "universe, fields, new_fields, removed_fields, expected_fields, expected_events",
        [
            (
                ["GBP=", "EUR="],
                ["ASK", "IRGPRC"],
                "PCTCHG_MTD",
                "ASK",
                ["PCTCHG_MTD", "IRGPRC"],
                ["Refresh", "Refresh", "Complete", "Update"],
            ),
            (
                "EUR=",
                "BID",
                "BID",
                "ASK",
                None,
                ["Refresh", "Complete", "Update"],
            ),
        ],
    )
    def test_add_and_remove_fields_to_existing_pricing_stream(
        self,
        open_session,
        universe,
        fields,
        new_fields,
        removed_fields,
        expected_fields,
        expected_events,
    ):
        triggered_events = []
        pricing_stream = pricing.Definition(
            universe=universe, fields=fields
        ).get_stream()
        add_callbacks_for_universe_stream(pricing_stream, triggered_events)
        pricing_stream.open()
        time.sleep(HALF_SECOND)
        pricing_stream.add_fields(new_fields)
        time.sleep(HALF_SECOND)
        pricing_stream.remove_fields(removed_fields)
        time.sleep(HALF_SECOND)
        snapshot = pricing_stream.get_snapshot()
        check_pricing_data(snapshot, universe, expected_fields)
        check_triggered_events(triggered_events, expected_events)
        assert (
            triggered_events.count("Complete") == 1
        ), f"Callback on_complete called more than once: {triggered_events.count(expected_events[1])}"

    @pytest.mark.caseid("C48673914")
    @allure.title("Add and remove instruments to existing stream")
    @pytest.mark.parametrize(
        "universe, fields, new_instrument, removed_instrument, expected_instrument",
        [
            (
                ["GBP=", "EUR="],
                ["ASK", "IRGPRC"],
                "USD=",
                "GBP=",
                ["EUR=", "USD="],
            ),
            (
                "EUR=",
                "BID",
                "USD=",
                "EUR=",
                "USD=",
            ),
        ],
    )
    def test_add_and_remove_instrument_to_existing_pricing_stream(
        self,
        open_desktop_session,
        universe,
        fields,
        new_instrument,
        removed_instrument,
        expected_instrument,
    ):
        triggered_events = []
        pricing_stream = pricing.Definition(
            universe=universe, fields=fields
        ).get_stream()
        add_callbacks_for_universe_stream(pricing_stream, triggered_events)
        pricing_stream.open()
        time.sleep(HALF_SECOND)
        pricing_stream.add_instruments(new_instrument)
        time.sleep(HALF_SECOND)
        pricing_stream.remove_instruments(removed_instrument)
        time.sleep(HALF_SECOND)
        snapshot = pricing_stream.get_snapshot()
        check_pricing_data(snapshot, expected_instrument)
        assert (
            triggered_events.count("Complete") == 1
        ), f"Callback on_complete called more than once: {triggered_events.count('Complete')}"

    @allure.title("Offstream contribute in pricing stream and check response")
    @pytest.mark.parametrize(
        "name,fields,message",
        [
            pytest.param(
                "TEST",
                {"BID": 240.83},
                "[1]: Contribution Accepted",
                id="positive_case",
            ),
            pytest.param(
                "TEST",
                {"INVAL": 240.83},
                "JSON Unexpected FID. Received 'INVAL' for key 'Fields'",
                id="negative_case",
            ),
        ],
    )
    @pytest.mark.caseid("C49042542")
    def test_offstream_contribution_pricing_stream_and_check_response(
        self, request, load_config, open_platform_session, name, fields, message
    ):
        event_list = []
        response = pricing.contribute(
            name=name,
            fields=fields,
            on_ack=lambda ack_msg, stream: on_ack(ack_msg, stream, event_list),
            on_error=lambda error_msg, stream: on_error(error_msg, stream, event_list),
            service="ATS_GLOBAL_1",
        )

        check_contrib_response(request, response, event_list, message)

    @allure.title("Offstream contribute async in pricing stream and check response")
    @pytest.mark.parametrize(
        "name,fields,message",
        [
            pytest.param(
                "TEST",
                {"BID": 777.83},
                "[1]: Contribution Accepted",
                id="positive_case",
            ),
            pytest.param(
                "TEST",
                {"INVAL": 777.83},
                "JSON Unexpected FID. Received 'INVAL' for key 'Fields'",
                id="negative_case",
            ),
        ],
    )
    @pytest.mark.caseid("C49042540")
    async def test_offstream_async_contribution_pricing_stream_and_check_response(
        self, request, load_config, open_platform_session_async, name, fields, message
    ):
        event_list = []
        response = await pricing.contribute_async(
            name=name,
            fields=fields,
            on_ack=lambda ack_msg, stream: on_ack(ack_msg, stream, event_list),
            on_error=lambda error_msg, stream: on_error(error_msg, stream, event_list),
            service="ATS_GLOBAL_1",
        )

        check_contrib_response(request, response, event_list, message)

    @allure.title("OnStream contribute in pricing stream and check snapshot")
    @pytest.mark.parametrize(
        "universe, fields, contrib_value, message",
        [
            pytest.param(
                "TEST",
                ["BID", "ASK", "OPEN_PRC"],
                {"BID": 503},
                "[1]: Contribution Accepted",
                id="positive_case",
            ),
            pytest.param(
                "TEST",
                "ASK",
                {"INVAL": 277.83},
                "JSON Unexpected FID. Received 'INVAL' for key 'Fields'",
                id="negative_case",
            ),
        ],
    )
    @pytest.mark.caseid("C49042532")
    def test_on_stream_contribution_pricing_stream(
        self,
        request,
        setup_direct_url,
        open_platform_session,
        universe,
        fields,
        contrib_value,
        message,
    ):
        event_list = []
        definition = pricing.Definition(
            universe=universe, fields=fields, service="ATS_GLOBAL_1"
        )
        stream = definition.get_stream()
        add_callback_for_contribution(stream, event_list)
        stream.open()
        time.sleep(HALF_SECOND)

        response = stream.contribute(
            universe, contrib_value, contrib_type=ContribType.REFRESH
        )
        if "positive_case" in request.node.callspec.id:
            snapshot = stream.get_snapshot()["BID"][0]
            assert (
                snapshot == contrib_value["BID"]
            ), f"Actual value {snapshot} != {contrib_value['BID']}"

        check_contrib_response(request, response, event_list, message)

    @allure.title("OnStream contribute async in pricing stream and check snapshot")
    @pytest.mark.parametrize(
        "universe, fields, contrib_value, message",
        [
            pytest.param(
                "TEST",
                ["BID", "ASK", "OPEN_PRC"],
                {"BID": 503},
                "[1]: Contribution Accepted",
                id="positive_case",
            ),
            pytest.param(
                "TEST",
                "ASK",
                {"INVAL": 277.83},
                "JSON Unexpected FID. Received 'INVAL' for key 'Fields'",
                id="negative_case",
            ),
        ],
    )
    @pytest.mark.caseid("C49042538")
    async def test_on_stream_async_contribution_pricing_stream(
        self,
        request,
        setup_direct_url,
        open_platform_session_async,
        universe,
        fields,
        contrib_value,
        message,
    ):
        event_list = []
        definition = pricing.Definition(
            universe=universe, fields=fields, service="ATS_GLOBAL_1"
        )
        stream = definition.get_stream()
        add_callback_for_contribution(stream, event_list)
        await stream.open_async()

        response = await stream.contribute_async(
            universe, contrib_value, contrib_type=ContribType.UPDATE
        )
        if "positive_case" in request.node.callspec.id:
            snapshot = stream.get_snapshot()["BID"][0]
            assert (
                snapshot == contrib_value["BID"]
            ), f"Actual value {snapshot} != {contrib_value['BID']}"

        check_contrib_response(request, response, event_list, message)

    @allure.title("Get pricing stream/data with scope validation")
    @pytest.mark.caseid("C53817129")
    @pytest.mark.parametrize(
        "universe,fields,scope",
        [
            (["GBP=", "EUR="], ["BID", "ASK", "HIGH_1"], "trapi.invalid_scope"),
        ],
    )
    def test_pricing_stream_data_with_scope_validation(
        self, create_session_with_scope, universe, fields, scope
    ):
        create_session_with_scope(scope=scope, acc="machine")
        with pytest.raises(ScopeError) as error:
            pricing.Definition(universe=universe, fields=fields).get_data()
        assert "Missing scopes" in str(error.value)

        with pytest.raises(ScopeError) as error:
            stream = pricing.Definition(
                universe=universe, fields=fields, api="streaming.pricing.endpoints.main"
            ).get_stream()
            stream.open()
        assert "Missing scopes" in str(error.value)
        time.sleep(1)

        assert (
            stream.open_state == OpenState.Closed
        ), "Pricing Stream is not closed after ScopeError Exception raised"



# coding: utf8

from enum import Enum, unique


@unique
class IndexCompoundingMethod(Enum):
    ADJUSTED_COMPOUNDED = "AdjustedCompounded"
    AVERAGE = "Average"
    COMPOUNDED = "Compounded"
    CONSTANT = "Constant"
    MEXICAN_COMPOUNDED = "MexicanCompounded"



import pytest

from refinitiv.data.content.esg.bulk._tools import (
    get_the_newest_file,
    get_init_archives,
    get_date_from_filename,
    is_correct_filename,
    get_filesets_with_delta_files,
    get_filesets_with_the_newest_init_file_and_delta_files,
    sorted_files,
    join_path_to_field,
    is_init_file,
)


@pytest.mark.parametrize(
    ("test_value", "expected_result"),
    [
        ("RFT-ESG-Raw-Full-SchemeB-Social-delta-2021-04-11.jsonl.gz", True),
        ("RFT-ESG-Raw-Full-SchemeB-Social-Delta-2021-04-11.jsonl.gz", True),
        ("RFT-ESG-Raw-Full-SchemeB-Social-Delta-2021-04-11", False),
        ("RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-03-12.jsonl.gz", True),
        ("RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-05-09.jsonl", True),
        ("RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-0-09.jsonl", False),
        ("RFT-ESG-Raw-Full-SchemeB-Social-Init.jsonl", False),
    ],
)
def test_is_correct_filename(test_value, expected_result):
    # given

    # when
    result = is_correct_filename(test_value)

    # then
    assert result == expected_result


@pytest.mark.parametrize(
    ("test_value", "expected_result"),
    [
        ("RFT-ESG-Raw-Full-SchemeB-Social-Jsonl-delta-2021-04-11.jsonl.gz", False),
        ("RFT-ESG-Raw-Full-SchemeB-Social-Jsonl-Delta-2021-04-11.jsonl.gz", False),
        ("RFT-ESG-Raw-Full-SchemeB-Social-Jsonl-init-2021-03-12.jsonl.gz", True),
        ("RFT-ESG-Raw-Full-SchemeB-Social-Jsonl-Init-2021-05-09.jsonl", True),
    ],
)
def test_is_init_file(test_value, expected_result):
    # given

    # when
    result = is_init_file(test_value)

    # then
    assert result == expected_result


@pytest.mark.parametrize(
    ("test_value", "expected_result"),
    [
        ("RFT-ESG-Raw-Full-SchemeB-Social-Delta-2021-04-11.jsonl.gz", "2021-04-11"),
        ("RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-03-12.jsonl.gz", "2021-03-12"),
        ("RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-05-09.jsonl", "2021-05-09"),
    ],
)
def test_get_date_from_filename(test_value, expected_result):
    # given

    # when
    result = get_date_from_filename(test_value)

    # then
    assert result == expected_result


@pytest.mark.parametrize(
    ("test_value", "expected_result"),
    [
        (
            [
                "RFT-ESG-Raw-Full-SchemeB-Social-Delta-2021-04-11.jsonl",
                "RFT-ESG-Raw-Full-SchemeB-Social-Delta-2021-05-08.jsonl",
                "RFT-ESG-Raw-Full-SchemeB-Social-Delta-2021-03-12.jsonl",
            ],
            "RFT-ESG-Raw-Full-SchemeB-Social-Delta-2021-05-08.jsonl",
        ),
        ([], None),
    ],
)
def test_get_the_newest_file(test_value, expected_result):
    # given

    # when
    result = get_the_newest_file(test_value)

    # then
    assert result == expected_result


@pytest.mark.parametrize(
    ("test_value", "expected_result"),
    [
        (
            [
                "RFT-ESG-Raw-Full-SchemeB-Social-Delta-2021-04-11.jsonl.gz",
                "RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-04-11.jsonl.gz",
                "RFT-ESG-Raw-Full-SchemeB-Social-Delta-2021-05-11.jsonl",
                "RFT-ESG-Raw-Full-SchemeB-Social-Delta-2019-11-12.jsonl",
                "RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-05-11.jsonl",
                "RFT-ESG-Raw-Full-SchemeB-Social-Init-2020-04-11.jsonl.gz",
                "RFT-ESG-Raw-Full-SchemeB-Social-Delta-2021-04-11.jsonl",
            ],
            [
                "RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-04-11.jsonl.gz",
                "RFT-ESG-Raw-Full-SchemeB-Social-Init-2020-04-11.jsonl.gz",
            ],
        ),
        (
            [
                "RFT-ESG-Raw-Full-SchemeB-Social-Delta-2021-04-11.jsonl.gz",
                "RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-04-11.jsonl.gz",
                "RFT-ESG-Raw-Full-SchemeB-Social-Delta-2021-05-11.jsonl",
                "RFT-ESG-Raw-Full-SchemeB-Social-Delta-2019-11-12.jsonl",
                "RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-05-11.jsonl",
                "RFT-ESG-Raw-Full-SchemeB-Social-Init.jsonl.gz",
                "RFT-ESG-Raw-Full-SchemeB-Social-Delta-2021-04-11.jsonl",
            ],
            [
                "RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-04-11.jsonl.gz",
            ],
        ),
        (
            [
                "RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-01-1.jsonl.gz",
                "RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-02-11.jsonl",
                "RFT-ESG-Raw-Full-SchemeB-Soci-Init-2021-03-11.jsonl.gz",
                "RFT-ESG-Raw-Full-SchemeB-Social--2021-04-11.jsonl.gz",
                "RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-05-11.jsonl.gz",
                "RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-06-11.jsonl.gz",
            ],
            [
                "RFT-ESG-Raw-Full-SchemeB-Soci-Init-2021-03-11.jsonl.gz",
                "RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-05-11.jsonl.gz",
                "RFT-ESG-Raw-Full-SchemeB-Social-Init-2021-06-11.jsonl.gz",
            ],
        ),
        ([], []),
    ],
)
def test_get_init_archives(test_value, expected_result):
    # given

    # when
    result = get_init_archives(test_value)

    # then
    assert set(result) == set(expected_result)


@pytest.mark.parametrize(
    ("test_value", "expected_result"),
    [
        (
            [
                {"name": "RFT-ESG-Scores-Wealth-Standard-Jsonl-Delta-2021-05-09"},
                {"name": "RFT-ESG-Scores-Wealth-Standard-Jsonl-init-2021-05-09"},
                {"name": "RFT-ESG-Scores-Wealth-Standard-Jsonl-delta-2021-05-09"},
            ],
            [
                {"name": "RFT-ESG-Scores-Wealth-Standard-Jsonl-Delta-2021-05-09"},
                {"name": "RFT-ESG-Scores-Wealth-Standard-Jsonl-delta-2021-05-09"},
            ],
        ),
    ],
)
def test_get_filesets_with_delta_files(test_value, expected_result):
    # given

    # when
    result = get_filesets_with_delta_files(test_value)

    # then
    assert result == expected_result


@pytest.mark.parametrize(
    ("test_value", "expected_result"),
    [
        (
            [
                {"name": "RFT-ESG-Scores-Wealth-Standard-Jsonl-Delta-2021-04-11"},
                {"name": "RFT-ESG-Scores-Wealth-Standard-Jsonl-Init-2021-03-09"},
                {"name": "RFT-ESG-Scores-Wealth-Standard-Jsonl-Delta-2021-03-09"},
                {"name": "RFT-ESG-Scores-Wealth-Standard-Jsonl-Delta-2021-04-22"},
                {"name": "RFT-ESG-Scores-Wealth-Standard-Jsonl-Init-2021-05-09"},
                {"name": "RFT-ESG-Scores-Wealth-Standard-Jsonl-Delta-2021-05-09"},
            ],
            [
                {"name": "RFT-ESG-Scores-Wealth-Standard-Jsonl-Init-2021-05-09"},
                {"name": "RFT-ESG-Scores-Wealth-Standard-Jsonl-Delta-2021-05-09"},
            ],
        ),
        ([], []),
    ],
)
def test_get_filesets_with_the_newest_init_file_and_delta_files(
    test_value, expected_result
):
    # given

    # when
    result = get_filesets_with_the_newest_init_file_and_delta_files(test_value)

    # then
    assert list(result) == expected_result


@pytest.mark.parametrize(
    ("test_value", "expected_result"),
    [
        ([], []),
        (
            [
                "RFT-ESG-Scores-Wealth-Standard-delta-2021-06-13.jsonl.gz",
                "RFT-ESG-Scores-Wealth-Standard-init-2021-06-13.jsonl.gz",
            ],
            [
                "RFT-ESG-Scores-Wealth-Standard-init-2021-06-13.jsonl.gz",
                "RFT-ESG-Scores-Wealth-Standard-delta-2021-06-13.jsonl.gz",
            ],
        ),
    ],
)
def test_sorted_files(test_value, expected_result):
    # given

    # when
    result = sorted_files(test_value)

    # then
    assert result == expected_result


@pytest.mark.parametrize(
    ("test_value", "expected_result"),
    [
        ([], ""),
        (["test_field_name"], "test_field_name"),
        (["test_field_name", "field"], "test_field_name.field"),
    ],
)
def test_join_path_to_field(test_value, expected_result):
    # given

    # when
    result = join_path_to_field(*test_value)

    # then
    assert result == expected_result



import allure
import pytest

from refinitiv.data import OpenState
from refinitiv.data._content_type import ContentType
from refinitiv.data.delivery import rdp_stream
from refinitiv.data.delivery._stream import stream_cxn_cache
from tests.integration.delivery.stream.conftest import (
    check_open_and_close_stream_state,
)
from tests.integration.delivery.stream.rdp_stream.conftest import (
    get_financial_contracts_stream,
    get_benchmark_stream,
    get_trading_analytics_stream,
    check_stream_view,
    check_stream_universe,
    add_callbacks,
)


@allure.suite("Delivery object - Stream")
@allure.feature("Delivery object - RDP Stream")
@allure.severity(allure.severity_level.CRITICAL)
class TestRdpStream:
    @allure.title("Open RDP stream, check stream parameters")
    @pytest.mark.parametrize(
        "rdp_stream_definition,expected_universe,expected_view",
        [
            [
                get_financial_contracts_stream,
                {
                    "instrumentType": "FxCross",
                    "instrumentDefinition": {
                        "instrumentTag": "USDAUD",
                        "fxCrossType": "FxSpot",
                        "fxCrossCode": "USDAUD",
                    },
                },
                [
                    "InstrumentTag",
                    "FxSpot_BidMidAsk",
                    "ErrorCode",
                    "Ccy1SpotDate",
                    "Ccy2SpotDate",
                ],
            ],
            [get_benchmark_stream, ["TSLA.O", "AAPL.O", "AMZN.O"], None],
            [get_trading_analytics_stream, [], None],
        ],
    )
    @pytest.mark.caseid("C39068745")
    def test_open_rdp_stream(
        self, open_session, rdp_stream_definition, expected_universe, expected_view
    ):
        stream = rdp_stream_definition()
        check_open_and_close_stream_state(stream)
        check_stream_universe(stream, expected_universe)
        check_stream_view(stream, expected_view)

    @allure.title("Open RDP stream with callbacks")
    @pytest.mark.parametrize(
        "expected_universe,expected_view",
        [
            [
                [
                    {
                        "instrumentType": "FxCross",
                        "instrumentDefinition": {
                            "instrumentTag": "USDAUD",
                            "fxCrossType": "FxSpot",
                            "fxCrossCode": "USDAUD",
                        },
                    }
                ],
                [
                    "InstrumentTag",
                    "FxSpot_BidMidAsk",
                    "ErrorCode",
                    "Ccy1SpotDate",
                    "Ccy2SpotDate",
                ],
            ]
        ],
    )
    @pytest.mark.caseid("C39068832")
    def test_open_rdp_stream_with_callbacks(
        self, open_session, expected_universe, expected_view
    ):
        stream = get_financial_contracts_stream()
        stream_log = []
        add_callbacks(stream, stream_log)

        check_open_and_close_stream_state(stream, wait_before_close=3)

        expected_event_types = ["response", "update"]
        for event in expected_event_types:
            assert event in stream_log, f"No event {event} received"

    @allure.title("Open RDP stream without required parameters")
    @pytest.mark.caseid("C39068923")
    def test_open_rdp_stream_without_required_parameters(self, open_session):
        with pytest.raises(TypeError, match="missing .* required positional argument"):
            rdp_stream.Definition(
                universe=[],
                view=None,
                service=None,
                api="streaming.trading-analytics.endpoints.redi",
            )

    @allure.title("Open RDP stream with extended params")
    @pytest.mark.caseid("C39068927")
    def test_open_rdp_stream_with_extended_params(self, open_session):
        stream = get_financial_contracts_stream(
            extended_params={"EXTENDED_PARAM": "EXTENDED_PARAM_VALUE"}
        )
        check_open_and_close_stream_state(stream)
        assert stream._extended_params["EXTENDED_PARAM"] == "EXTENDED_PARAM_VALUE"

    @allure.title("Open async RDP stream")
    @pytest.mark.caseid("C39068929")
    async def test_open_rdp_stream_async(self, open_session_async):
        stream = get_financial_contracts_stream()
        await stream.open_async()
        assert stream.open_state == OpenState.Opened, "Stream is not in opened state"
        stream.close()
        assert stream.open_state == OpenState.Closed, "Stream is not in closed state"

    @allure.title("Close RDP stream by closing session")
    @pytest.mark.caseid("C39068931")
    def test_stream_closes_after_session_close(self, open_desktop_session):
        session = open_desktop_session
        stream = get_financial_contracts_stream()
        stream.open()
        session.close()

        assert stream.open_state == OpenState.Closed, "Stream is not in closed state"
        is_cxn_alive = stream_cxn_cache.is_cxn_alive(
            session, ContentType.STREAMING_CONTRACTS
        )
        assert not is_cxn_alive, f"Connection is alive"

    @allure.title("Verify RDP stream is opened and closed with ContextManager")
    @pytest.mark.parametrize(
        "rdp_stream_definition",
        [get_financial_contracts_stream],
    )
    @pytest.mark.caseid("C44023017")
    def test_open_rdp_stream_with_context_manager(
        self, open_session, rdp_stream_definition
    ):
        stream = rdp_stream_definition()
        with stream:
            assert (
                stream.open_state == OpenState.Opened
            ), "RDP Stream is not opened in ContextManager"
        assert (
            stream.open_state == OpenState.Closed
        ), "RDP Stream is not closed outside ContextManager"

    @allure.title(
        "Verify RDP stream is opened and closed with asynchronous ContextManager"
    )
    @pytest.mark.parametrize(
        "rdp_stream_definition",
        [
            get_financial_contracts_stream,
        ],
    )
    @pytest.mark.caseid("C44023018")
    async def test_open_rdp_stream_with_async_context_manager(
        self, open_session_async, rdp_stream_definition
    ):
        stream = rdp_stream_definition()
        async with stream:
            assert (
                stream.open_state == OpenState.Opened
            ), "RDP Stream is not opened in ContextManager"
        assert (
            stream.open_state == OpenState.Closed
        ), "RDP Stream is not closed outside ContextManager"



import asyncio
from time import sleep

import allure
import pytest

from refinitiv.data import OpenState
from refinitiv.data._content_type import ContentType
from refinitiv.data.delivery import omm_stream
from refinitiv.data.delivery._stream import stream_cxn_cache
from tests.integration.delivery.stream.conftest import (
    check_open_and_close_stream_state,
)
from tests.integration.delivery.stream.omm_stream.conftest import (
    open_deployed_session_for_omm_stream,
    open_platform_session_for_omm_stream,
    check_triggered_events,
    check_stream_view,
    check_stream_universe,
    check_extended_params,
    get_error_event_text_msg,
    on_ack,
    on_error,
)
from tests.integration.helpers import add_callbacks, assert_error
from tests.integration.helpers import check_contrib_response


@allure.suite("Delivery object - Stream")
@allure.feature("Delivery object - OMM stream")
@allure.severity(allure.severity_level.CRITICAL)
class TestOMMStream:
    @allure.title(
        "Create OMM stream object with all callbacks, check stream parameters"
    )
    @pytest.mark.parametrize(
        "open_custom_session,name,fields,api,domain,service,expected_events",
        [
            (
                open_platform_session_for_omm_stream,
                "GBP=",
                ["BID", "ASK"],
                "streaming.pricing.endpoints.main",
                "MarketPrice",
                None,
                ["Refresh", "Complete", "Update"],
            ),
            (
                open_platform_session_for_omm_stream,
                "GBP=",
                [],
                "streaming.pricing.endpoints.direct-websocket",
                "MarketPrice",
                "ELEKTRON_DD",
                ["Refresh", "Complete", "Update"],
            ),
            (
                open_deployed_session_for_omm_stream,
                "BB.TO",
                None,
                None,
                "MarketByPrice",
                "ELEKTRON_DD",
                ["Refresh"],
            ),
        ],
    )
    @pytest.mark.caseid("37805456")
    def test_create_omm_stream_with_all_callbacks(
        self,
        open_custom_session,
        name,
        fields,
        api,
        domain,
        service,
        expected_events,
    ):
        session = open_custom_session()
        triggered_events = []
        stream = omm_stream.Definition(
            name=name, api=api, service=service, fields=fields, domain=domain
        ).get_stream(session)
        add_callbacks(stream, triggered_events)
        check_open_and_close_stream_state(stream, wait_before_close=6)
        session.close()
        check_triggered_events(triggered_events, expected_events)
        check_stream_universe(name, triggered_events)
        check_stream_view(fields, triggered_events)

    @allure.title("Listening two streams and keep retrieving data for active stream")
    @pytest.mark.parametrize(
        "name,fields01,fields02,expected_events",
        [("GBP=", "BID", "ASK", ["Refresh", "Complete"])],
    )
    @pytest.mark.caseid("39516504")
    async def test_keep_retrieving_data_for_two_streams(
        self, name, fields01, fields02, open_session_async, expected_events
    ):
        triggered_events_01 = []
        stream_01 = omm_stream.Definition(name=name, fields=fields01).get_stream()
        add_callbacks(stream_01, triggered_events_01)
        triggered_events_02 = []
        stream_02 = omm_stream.Definition(name=name, fields=fields02).get_stream()
        add_callbacks(stream_02, triggered_events_02)

        await asyncio.gather(stream_01.open_async(), stream_02.open_async())
        sleep(5)
        check_triggered_events(triggered_events_01, expected_events)
        check_triggered_events(triggered_events_02, expected_events)

    @allure.title(
        "Stop listening closed OMM stream and keep retrieving data for active stream"
    )
    @pytest.mark.parametrize(
        "name01,name02,expected_events",
        [("EUR=", "GBP=", ["Refresh", "Complete", "Update"])],
    )
    @pytest.mark.caseid("37805459")
    async def test_keep_retrieving_data_for_active_stream(
        self, name01, name02, expected_events, open_session_async
    ):
        stream_01 = omm_stream.Definition(name=name01).get_stream()
        stream_01.open()

        triggered_events_02 = []
        stream_02 = omm_stream.Definition(name=name02).get_stream()
        add_callbacks(stream_02, triggered_events_02)
        stream_02.open()
        sleep(6)
        stream_01.close()
        assert stream_01.open_state == OpenState.Closed, "Stream is not in closed state"

        check_triggered_events(triggered_events_02, expected_events)
        stream_02.close()

    @allure.title("Open OMM stream without required parameters")
    @pytest.mark.caseid("39516358")
    def test_open_omm_stream_without_required_parameters(self):
        with pytest.raises(TypeError) as error:
            omm_stream.Definition().get_stream()
        assert_error(error, "name")

    @allure.title("Get an error for OMM stream with invalid name")
    @pytest.mark.parametrize(
        "name,error_msg", [("INVALID", "The record could not be found")]
    )
    @pytest.mark.caseid("37805461")
    def test_open_omm_stream_with_invalid_params(self, open_session, name, error_msg):
        stream = omm_stream.Definition(name=name).get_stream()
        triggered_events = []
        add_callbacks(stream, triggered_events)
        check_open_and_close_stream_state(stream, wait_before_close=3)
        assert error_msg in get_error_event_text_msg(triggered_events)

    @allure.title("Open OMM stream with extended params")
    @pytest.mark.parametrize(
        "name,fields,extended_params", [("GBP=", ["ASK"], {"View": ["BID"]})]
    )
    @pytest.mark.caseid("37805458")  # review case with w/o defining fields
    def test_open_omm_stream_with_extended_params(
        self, open_session, name, fields, extended_params
    ):
        stream = omm_stream.Definition(
            name=name, fields=fields, extended_params=extended_params
        ).get_stream()
        triggered_events_list = []
        add_callbacks(stream, triggered_events_list)
        check_open_and_close_stream_state(stream, wait_before_close=6)
        check_extended_params(stream, extended_params, triggered_events_list)

    @allure.title("Open async OMM stream")
    @pytest.mark.parametrize("name,fields", [("GBP=", ["BID", "ASK"])])
    @pytest.mark.caseid("39516353")
    async def test_open_omm_stream_async(self, open_session_async, name, fields):
        stream = omm_stream.Definition(name=name, fields=fields).get_stream()
        await stream.open_async()
        assert stream.open_state == OpenState.Opened, "Stream is not in opened state"
        stream.close()
        assert stream.open_state == OpenState.Closed, "Stream is not in closed state"

    @allure.title("Close OMM stream by closing session")
    @pytest.mark.parametrize("name,fields", [("GBP=", ["BID", "ASK"])])
    @pytest.mark.caseid("39516313")
    def test_stream_closes_after_session_close(
        self, open_desktop_session, name, fields
    ):
        session = open_desktop_session
        stream = omm_stream.Definition(name, fields=fields).get_stream()
        stream.open()
        session.close()

        assert stream.open_state == OpenState.Closed, "Stream is not in closed state"
        is_cxn_alive = stream_cxn_cache.is_cxn_alive(
            session, ContentType.STREAMING_PRICING
        )
        assert not is_cxn_alive, f"Connection is alive"

    @allure.title("Verify OMM stream is opened and closed with ContextManager")
    @pytest.mark.parametrize("name,fields", [("GBP=", ["BID", "ASK"])])
    @pytest.mark.caseid("C44023016")
    def test_open_omm_stream_with_context_manager(self, open_session, name, fields):
        stream = omm_stream.Definition(name=name, fields=fields).get_stream()
        with stream:
            assert (
                stream.open_state == OpenState.Opened
            ), "OMM Stream is not opened in ContextManager"
        assert (
            stream.open_state == OpenState.Closed
        ), "OMM Stream is not closed outside ContextManager"

    @allure.title(
        "Verify OMM stream is opened and closed with asynchronous ContextManager"
    )
    @pytest.mark.parametrize("name,fields", [("GBP=", ["BID", "ASK"])])
    @pytest.mark.caseid("C44023015")
    async def test_open_omm_stream_with_async_context_manager(
        self, open_session_async, name, fields
    ):
        stream = omm_stream.Definition(name=name, fields=fields).get_stream()
        async with stream:
            assert (
                stream.open_state == OpenState.Opened
            ), "OMM Stream is not opened in ContextManager"
        assert (
            stream.open_state == OpenState.Closed
        ), "OMM Stream is not closed outside ContextManager"

    @allure.title("Offstream contribute in OMM stream and check response")
    @pytest.mark.parametrize(
        "name,fields,message",
        [
            pytest.param(
                "TEST",
                {"BID": 240.83},
                "[1]: Contribution Accepted",
                id="positive_case",
            ),
            pytest.param(
                "TEST",
                {"INVAL": 240.83},
                "JSON Unexpected FID. Received 'INVAL' for key 'Fields'",
                id="negative_case",
            ),
        ],
    )
    @pytest.mark.caseid("C48464965")
    def test_offstream_contribution_omm_stream_and_check_response(
        self, request, load_config, open_platform_session, name, fields, message
    ):
        event_list = []
        response = omm_stream.contribute(
            name=name,
            fields=fields,
            on_ack=lambda ack_msg, stream: on_ack(ack_msg, stream, event_list),
            on_error=lambda error_msg, stream: on_error(error_msg, stream, event_list),
            service="ATS_GLOBAL_1",
        )

        check_contrib_response(request, response, event_list, message)

    @allure.title("Offstream async contribute in OMM stream and check response")
    @pytest.mark.parametrize(
        "name,fields,message",
        [
            pytest.param(
                "TEST",
                {"BID": 240.83},
                "[1]: Contribution Accepted",
                id="positive_case",
            ),
            pytest.param(
                "TEST",
                {"INVAL": 240.83},
                "JSON Unexpected FID. Received 'INVAL' for key 'Fields'",
                id="negative_case",
            ),
        ],
    )
    @pytest.mark.caseid("C48464966")
    async def test_offstream_async_contribution_omm_stream_and_check_response(
        self, request, load_config, open_platform_session_async, name, fields, message
    ):
        event_list = []
        response = await omm_stream.contribute_async(
            name=name,
            fields=fields,
            on_ack=lambda ack_msg, stream: on_ack(ack_msg, stream, event_list),
            on_error=lambda error_msg, stream: on_error(error_msg, stream, event_list),
            service="ATS_GLOBAL_1",
        )

        check_contrib_response(request, response, event_list, message)



import logging
import random

import pytest

from refinitiv.data.delivery import cfs
from tests.integration.constants_list import ISO_DATE_FORMAT
from tests.integration.delivery.cfs.conftest import (
    convert_string_to_date_format,
)


def get_random_file_name_from_fileset(fileset):
    response = cfs.files.Definition(fileset.id, page_size=10).get_data()
    random_file = random.choice(response.data.raw["value"])
    return random_file["filename"]


def check_single_file_received(response):
    files_count = len(response.data.raw["value"])
    assert (
        files_count == 1
    ), f"Response contains more than 1 file. Number of files: {files_count}, response:  \n {response.data.raw['value']}"


def check_all_files_contain_fileset_id(response, expected_fileset_id):
    files = response.data.raw["value"]
    for file in files:
        assert (
            file["filesetId"] == expected_fileset_id
        ), f"Found file which does not belong to defined fileset id {expected_fileset_id}: \n {file}"


def check_file_name_in_response(response, expected_file_name):
    files = response.data.raw["value"]
    for file in files:
        assert (
            file["filename"] == expected_file_name
        ), f"Found file with unexpected name: \n {file} \n expected name: {expected_file_name}"


def check_files_in_response_created_after_date(response, expected_date):
    date = convert_string_to_date_format(expected_date, ISO_DATE_FORMAT)
    files = response.data.raw["value"]
    for file in files:
        try:
            file_created_date = convert_string_to_date_format(
                file["created"], ISO_DATE_FORMAT
            )
            assert (
                file_created_date >= date
            ), f"found fileset modified before the required date {date}: \n {file}"
        except KeyError:
            logging.warning(f"The fileset does not have 'modified' field: {file}")


def check_files_in_response_modified_after_date(response, expected_date):
    date = convert_string_to_date_format(expected_date, ISO_DATE_FORMAT)
    files = response.data.raw["value"]
    for file in files:
        try:
            file_modified_date = convert_string_to_date_format(
                file["modified"], ISO_DATE_FORMAT
            )
            assert (
                file_modified_date >= date
            ), f"found fileset modified before the required date {date}: \n {file}"
        except KeyError:
            logging.warning(f"The fileset does not have 'modified' field: {file}")


def check_files_in_two_responses_are_not_the_same(response1, response2):
    for file in response1.data.raw["value"]:
        assert (
            file not in response2.data.raw["value"]
        ), f"Fileset from first request found in list of filesets in the second request: \n {file}"


@pytest.fixture(scope="function")
def get_random_fileset():
    filesets_response = cfs.file_sets.Definition("bulk-ESG", page_size=100).get_data()
    return random.choice(filesets_response.data.file_sets)



import allure
import pytest

from refinitiv.data._errors import RDError
from refinitiv.data.content import search
from refinitiv.data.content.search import Views
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.helpers import (
    check_response_status,
    check_non_empty_response_data,
    get_async_response_from_definition,
)


@allure.suite("Content object - Search Metadata")
@allure.feature("Content object - Search Metadata")
@allure.severity(allure.severity_level.CRITICAL)
class TestMetadata:
    @allure.title("Send metadata request with valid view parameter and get data")
    @pytest.mark.caseid("36136187")
    @pytest.mark.smoke
    @pytest.mark.parametrize("view", [(Views.PEOPLE), ("People")])
    def test_send_metadata_request_with_valid_view_parameter_and_get_data(
        self, open_session, view
    ):
        response = search.metadata.Definition(view=view).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)

    @allure.title("Send metadata request with invalid view parameter")
    @pytest.mark.caseid("36136196")
    @pytest.mark.parametrize(
        "view,expected_error",
        [
            (
                "invalid value",
                "Error code 400 | Validation error: validation failure list:",
            )
        ],
    )
    def test_send_metadata_request_with_invalid_view_parameter(
        self, open_session, view, expected_error
    ):
        with pytest.raises(RDError) as error:
            search.metadata.Definition(view=view).get_data()
        assert expected_error in str(error.value)

    @allure.title(
        "Send metadata request with valid view parameter and get data asynchronously"
    )
    @pytest.mark.caseid("36136243")
    @pytest.mark.parametrize("view", [Views.CMO_INSTRUMENTS])
    async def test_send_metadata_request_with_valid_view_parameter_and_get_data_asynchronously(
        self, open_session_async, view
    ):
        response = await get_async_response_from_definition(
            search.metadata.Definition(view=view)
        )
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)

    @allure.title("Send metadata request when session is not opened")
    @pytest.mark.caseid("36136244")
    @pytest.mark.parametrize("view", [Views.ORGANISATIONS])
    def test_send_metadata_request_when_session_is_not_opened(self, open_session, view):
        session = open_session
        session.close()
        definition = search.metadata.Definition(view=view)
        with pytest.raises(
            ValueError, match="Session is not opened. Can't send any request"
        ):
            definition.get_data()



import pytest

import refinitiv.data as rd
from refinitiv.data import get_config
from refinitiv.data._core.session import GrantPassword


def test_session_definition_session_name():
    config = get_config()
    config.set_param("sessions.desktop.workspace.app-key", "mocked-app-key")

    definition = rd.session.Definition(name="desktop.workspace")
    session = definition.get_session()

    assert session.name == "workspace"


def test_desktop_session_definition_scope():
    expected_attributes = "get_session"
    config = get_config()
    config.set_param("sessions.desktop.workspace.app-key", "mocked-app-key")

    definition = rd.session.Definition(name="desktop.workspace")

    assert expected_attributes in dir(definition)


def test_definition_from_config(write_project_config):
    from refinitiv.data import _configure as configure

    configure.reload()

    app_key_mock = "MOCK-app_key"
    grant_mock = GrantPassword("MOCK-username", "MOCK-password")
    signon_control_mock = "MOCK-signon_control"
    deployed_platform_host_mock = "MOCK-deployed_platform_host"
    deployed_platform_username_mock = "MOCK-deployed_platform_username"
    dacs_position_mock = "MOCK-dacs_position"
    dacs_application_id_mock = "MOCK-dacs_application_id"

    d = {
        "sessions": {
            "platform": {
                "custom-session": {
                    "app-key": app_key_mock,
                    "username": grant_mock.get_username(),
                    "password": grant_mock.get_password(),
                    "signon_control": signon_control_mock,
                    "realtime-distribution-system": {
                        "url": deployed_platform_host_mock,
                        "dacs": {
                            "username": deployed_platform_username_mock,
                            "application-id": dacs_application_id_mock,
                            "position": dacs_position_mock,
                        },
                    },
                }
            }
        },
    }
    write_project_config(d)
    configure.reload()
    definition = rd.session.Definition("platform.custom-session")
    session = definition.get_session()
    assert session.app_key == app_key_mock
    assert session._grant is not grant_mock
    assert session._grant.get_username() == grant_mock.get_username()
    assert session._grant.get_password() == grant_mock.get_password()
    assert session._take_signon_control == signon_control_mock
    assert session._deployed_platform_host == deployed_platform_host_mock

    assert session.name == "custom-session"
    assert (
        session._dacs_params.deployed_platform_username
        == deployed_platform_username_mock
    )
    assert session._dacs_params.dacs_position == dacs_position_mock
    assert session._dacs_params.dacs_application_id == dacs_application_id_mock


def test_definition_from_config_without_error_for_desktop_session(write_project_config):
    from refinitiv.data import _configure as configure

    configure.reload()

    config = {
        "sessions": {
            "default": "desktop.custom-session",
            "desktop": {
                "custom-session": {
                    "app-key": "DEFAULT_WORKSPACE_APP_KEY",
                }
            },
        },
    }
    write_project_config(config)
    configure.reload()
    try:
        definition = rd.session.Definition()
        definition.get_session()
    except Exception as e:
        assert False, str(e)
    else:
        assert True


def test_definition_from_config_error_invalid_session_type(write_project_config):
    from refinitiv.data import _configure as configure

    configure.reload()

    config = {
        "sessions": {
            "default": "desktop.custom-session",
            "desktop": {
                "custom-session": {
                    "app-key": "DEFAULT_WORKSPACE_APP_KEY",
                }
            },
        },
    }
    write_project_config(config)
    configure.reload()
    with pytest.raises(
        Exception, match="Cannot get session type by value:invalid-type"
    ):
        definition = rd.session.Definition("invalid-type.default")
        definition.get_session()


def test_definition_from_config_specific_apis(write_project_config):
    from refinitiv.data import _configure as configure

    configure.reload()

    app_key_mock = "MOCK-app_key"
    grant_mock = GrantPassword("MOCK-username", "MOCK-password")
    signon_control_mock = "MOCK-signon_control"

    d = {
        "sessions": {
            "platform": {
                "custom-session": {
                    "app-key": app_key_mock,
                    "username": grant_mock.get_username(),
                    "password": grant_mock.get_password(),
                    "signon_control": signon_control_mock,
                    "apis": {"test_key": "test_value"},
                }
            }
        },
    }
    write_project_config(d)
    configure.reload()
    definition = rd.session.Definition("platform.custom-session")
    session = definition.get_session()

    assert session.name == "custom-session"
    test_value = session.config.get("apis.test_key")
    assert test_value == "test_value"



from refinitiv.data.content.ipa._curves._cross_currency_curves._definitions._delete import (
    DeleteRequest,
)


def test_curve_definition_delete_request():
    # given
    expected_dict = {"id": "string"}

    # when
    testing_obj = DeleteRequest(
        id="string",
    )

    # then
    assert testing_obj.get_dict() == expected_dict

    assert testing_obj.id == "string"



from enum import Enum, unique


@unique
class InstrumentType(Enum):
    BOND = "Bond"
    BOND_FUTURES = "BondFutures"
    CALENDAR_SPREAD = "CalendarSpread"
    CREDIT_DEFAULT_SWAP = "CreditDefaultSwap"
    CROSS_CURRENCY_SWAP = "CrossCurrencySwap"
    DEPOSIT = "Deposit"
    FRA = "Fra"
    FUTURES = "Futures"
    FX_FORWARD = "FxForward"
    FX_SPOT = "FxSpot"
    INTER_PRODUCT_SPREAD = "InterProductSpread"
    INTEREST_RATE_SWAP = "InterestRateSwap"
    OVERNIGHT_INDEX_SWAP = "OvernightIndexSwap"
    TENOR_BASIS_SWAP = "TenorBasisSwap"



import datetime
import logging
import logging.handlers
import os

import allure
import pytest

import refinitiv.data as rd
import refinitiv.data._log as rdp_logging


class Record(object):
    def __init__(self, name) -> None:
        super().__init__()
        self.name = name


@allure.suite("Log reporter")
@allure.feature("Log reporter")
@allure.severity(allure.severity_level.CRITICAL)
class TestLogger:
    @allure.title("Total count of all rotated log files")
    @pytest.mark.caseid("")
    def test_total_count_of_all_rotated_log_files(
        self, enabled_log, open_desktop_session
    ):
        max_files = 2

        session = open_desktop_session
        rd.get_config().set_param(
            param=f"logs.transports.file.maxFiles", value=max_files
        )
        logger = session.logger()

        file_handler = None
        for hdlr in logger.handlers:
            if isinstance(hdlr, logging.handlers.RotatingFileHandler):
                file_handler = hdlr
                break

        for i in range(max_files):
            logger.error(f"error message {i}")
            file_handler.doRollover()

        filepath = rdp_logging._filenamer(file_handler.baseFilename + f".{max_files}")
        assert os.path.exists(filepath)

        filepath = rdp_logging._filenamer(
            file_handler.baseFilename + f".{max_files + 1}"
        )
        assert not os.path.exists(filepath)

    @allure.title("Logs are being written by the logger contain the module name")
    @pytest.mark.caseid("")
    def test_logs_are_being_written_by_the_logger_contain_the_module_name(
        self, enabled_log
    ):
        logger = rdp_logging.create_logger("session:desktop")
        logger.error("error message")
        logger.info("info message")
        now_ = datetime.datetime.now()
        date_ = now_.strftime("%Y-%m-%d")

        assert "session:desktop" in logger.name
        assert f"{date_}" in logger.root.root.handlers[2].records[0].asctime
        assert "ERROR" in logger.root.root.handlers[2].records[0].levelname
        assert "INFO" in logger.root.root.handlers[2].records[1].levelname

    @allure.title("All levels below or equal VALUE are logged")
    @pytest.mark.parametrize(
        "string_value,int_value",
        [
            ("trace", 5),
            ("debug", 10),
            ("info", 20),
            ("warn", 30),
            ("error", 40),
        ],
    )
    @pytest.mark.caseid(" ")
    def test_all_levels_below_or_equal_value_are_logged(
        self, string_value, int_value, enabled_log, open_desktop_session
    ):
        session = open_desktop_session
        session.set_log_level(int_value)

        log_levels = [(string_value, int_value)]
        value = rdp_logging.convert_log_level(string_value)

        logger = session.logger()
        levels_sum = 0
        target_levels_sum = 0
        start_to_sum = False
        for level_str, level_int in log_levels:
            log_level = rdp_logging.convert_log_level(level_int)
            logger.log(log_level, "log message")

            levels_sum += session.logger().level

            if log_level == value:
                start_to_sum = True

            if start_to_sum:
                target_levels_sum += log_level

        assert (
            target_levels_sum == levels_sum
        ), f"target: {target_levels_sum}, exists: {levels_sum}"

    @allure.title("Logs are not written at all")
    @pytest.mark.caseid("")
    def test_logs_are_not_written(self, enabled_log, open_platform_session):
        session = open_platform_session
        session.set_log_level("silent")
        logger = session.logger()
        logger.error("error message")

        assert len(logger.root.root.handlers[2].records) == 0

    @allure.title("Default log level is info")
    @pytest.mark.caseid("")
    def test_get_default_log_level(self, enabled_log, open_platform_session):
        session = open_platform_session
        logger = session.logger()
        logger.error("error message")
        logger.warning("warning message")
        logger.info("info message")
        logger.debug("debug message")

        assert logger.level == 20

    @allure.title("Date, time, and process ID are added to the log file name")
    @pytest.mark.caseid("")
    def test_date_time_process_id_are_added_to_the_log_file_name(
        self, enabled_log, open_desktop_session
    ):
        default_name = rd.get_config().get_param(param=f"logs.transports.file.name")
        logger = open_desktop_session.logger()
        logger.error("error message")

        filename = logger.handlers[0].filename
        date, time, process_id, *name = filename.split("-")
        name = "-".join(name)
        assert date and int(date)
        assert time and int(time)
        assert process_id and int(process_id)
        assert name == default_name

    @allure.title("Check log size")
    @pytest.mark.caseid("")
    def test_check_log_size(self, enabled_log, open_desktop_session):
        default_file_size = rd.get_config().get_param(
            param=f"logs.transports.file.size"
        )
        logger = open_desktop_session.logger()

        file_handler = None
        stream_handler = None
        for hdlr in logger.handlers:
            if isinstance(hdlr, logging.handlers.RotatingFileHandler):
                file_handler = hdlr
            if isinstance(hdlr, logging.StreamHandler):
                stream_handler = hdlr

        logger.removeHandler(stream_handler)

        expected_filesize = rdp_logging.convert_filesize(default_file_size)
        max_bytes = file_handler.maxBytes

        start = 0
        stop = expected_filesize // 100
        for i in range(start, stop + 1):
            logger.info("a" * 30)

        filepath = rdp_logging._filenamer(file_handler.baseFilename + ".1")
        filesize = os.path.getsize(filepath)

        assert expected_filesize == max_bytes, max_bytes
        assert filesize <= expected_filesize, f"Actual file size: {filesize}"
        assert default_file_size == "10M", default_file_size
        open_desktop_session.close()

    @allure.title("Only those logs that matched by filter are written")
    @pytest.mark.parametrize(
        "value,record",
        [
            ("session*,module:b,module:c", "module:c"),
            ("*,-module:a", "module:b"),
            ("module:*,-module:b,-module:c", "module:a"),
        ],
    )
    @pytest.mark.caseid("")
    def test_only_those_logs_that_matched_by_filter_are_written(
        self, value, record, enabled_log
    ):
        rd.get_config().set_param(param=f"logs.filter", value=value)
        filterer = rdp_logging.make_filter(value)
        record = Record(record)
        result = filterer(record)
        assert result, f"record={record.name}, filter_by={value}"



from inspect import signature
import refinitiv.data.content.ipa.financial_contracts as rdf
from refinitiv.data.delivery._data._data_provider_factory import make_provider
from tests.unit.conftest import StubSession
import pytest

from tests.unit.conftest import (
    remove_dunder_methods,
    remove_private_attributes,
    has_property_names_in_class,
    get_property_names,
)


def test_ipa_cross_definition_attributes():
    expected_attributes = ["get_data", "get_data_async", "get_stream"]
    testing_attributes = dir(rdf.cross.Definition)
    testing_attributes = remove_dunder_methods(testing_attributes)
    testing_attributes = remove_private_attributes(testing_attributes)
    assert expected_attributes == testing_attributes


def test_ipa_cross_attributes():
    expected_attributes = [
        "BuySell",
        "Definition",
        "FxCrossType",
        "FxLegType",
        "FxPoint",
        "FxSwapCalculationMethod",
        "ImpliedDepositDateConvention",
        "LegDefinition",
        "PriceSide",
        "PricingParameters",
    ]
    testing_attributes = dir(rdf.cross)
    testing_attributes = remove_dunder_methods(testing_attributes)
    testing_attributes = remove_private_attributes(testing_attributes)
    assert expected_attributes == testing_attributes, testing_attributes


@pytest.mark.parametrize(
    argnames="input_data",
    ids=["Definition", "LegDefinition", "PricingParameters"],
    argvalues=[
        (
            rdf.cross._fx_cross_definition.FxCrossInstrumentDefinition,
            {
                "instrument_tag": "instrument_tag",
                "legs": [rdf.cross.LegDefinition()],
                "fx_cross_type": rdf.cross.FxCrossType.FX_FORWARD,
                "fx_cross_code": "fx_cross_code",
                "ndf_fixing_settlement_ccy": "ndf_fixing_settlement_ccy",
                "reference_spot_rate": 6.6,
                "traded_cross_rate": 7.7,
                "traded_swap_points": 8.8,
            },
        ),
        (
            rdf.cross.LegDefinition,
            {
                "start_date": "start_date",
                "end_date": "end_date",
                "tenor": "tenor",
                "leg_tag": "leg_tag",
                "deal_ccy_buy_sell": rdf.cross.BuySell.BUY,
                "fx_leg_type": rdf.cross.FxLegType.FX_SPOT,
                "contra_amount": 7.7,
                "contra_ccy": "contra_ccy",
                "deal_amount": 9.9,
                "deal_ccy": "deal_ccy",
                "start_tenor": "start_tenor",
            },
        ),
        (
            rdf.cross.PricingParameters,
            {
                "fx_swap_calculation_method": rdf.cross.FxSwapCalculationMethod.FX_SWAP,
                "implied_deposit_date_convention": rdf.cross.ImpliedDepositDateConvention.FX_MARKET_CONVENTION,
                "price_side": rdf.cross.PriceSide.ASK,
                "adjust_all_deposit_points_to_cross_calendars": True,
                "adjust_all_swap_points_to_cross_calendars": True,
                "ignore_ref_ccy_holidays": True,
                "market_data_date": "market_data_date",
                "report_ccy": "report_ccy",
                "use_direct_quote": True,
                "valuation_date": "valuation_date",
            },
        ),
    ],
)
def test_parameter(input_data):
    cls, kwargs = input_data
    args_names = list(kwargs.keys())
    inst = cls(**kwargs)

    s = signature(cls.__init__)
    # +1 for (self)
    assert len(s.parameters) == (
        len(args_names) + 1
    ), f"csl={cls}{str(set(get_property_names(cls)) - set(args_names))}"

    assert has_property_names_in_class(cls, args_names), set(args_names) ^ set(
        get_property_names(cls)
    )

    for k, v in kwargs.items():
        attr = getattr(inst, k)
        assert attr == v, k


def test_leg_definition_start_date():
    # given
    definition = rdf.cross.LegDefinition(
        tenor="1M",
        start_date="2021-01-01",
        end_date="2021-02-01",
    )

    # then
    assert definition.start_date == "2021-01-01"


def test_leg_definition_start_tenor():
    # given
    definition = rdf.cross.LegDefinition(
        start_tenor="1M",
        tenor="1M",
        end_date="2021-02-01",
    )

    # then
    assert definition.start_tenor == "1M"


def test_request_body():
    # given
    expected_body = {
        "fields": ["1", "2"],
        "universe": [
            {
                "instrumentDefinition": {
                    "fxCrossCode": "fx_cross_code",
                    "legs": [{"startDate": "start_date"}],
                },
                "extended_params": "extended_params",
                "instrumentType": "FxCross",
                "pricingParameters": {"marketDataDate": "market_data_date"},
            }
        ],
    }
    definition = rdf.cross.Definition(
        fx_cross_code="fx_cross_code",
        fields=["1", "2"],
        legs=[rdf.cross.LegDefinition(start_date="start_date")],
        pricing_parameters=rdf.cross.PricingParameters(
            market_data_date="market_data_date"
        ),
        extended_params={"extended_params": "extended_params"},
    )

    # when
    content_type = definition._kwargs["__content_type__"]
    provider = make_provider(content_type)
    request = provider.request.create(StubSession(), "url", **definition._kwargs)

    # then
    testing_body = request.json
    assert testing_body == expected_body, expected_body



from typing import Optional, List

from ......_tools import try_copy_to_list
from ._instrument_description import CrossCurrencyInstrumentDescription
from ._fx_forward_instrument_description import FxForwardInstrumentDescription
from ._fx_spot_instrument_description import FxSpotInstrumentDescription
from ...._object_definition import ObjectDefinition


class CrossCurrencyConstituentsDescription(ObjectDefinition):
    """
    Creates the Cross Currency curve definition with the definition provided.

    Parameters
    ----------
    cross_currency_swaps : list of CrossCurrencyInstrumentDescription, optional

    fx_forwards : list of FxForwardInstrumentDescription, optional

    fx_spot :  FxSpotInstrumentDescription, optional

    interest_rate_swaps : list of CrossCurrencyInstrumentDescription, optional

    overnight_index_swaps : list of CrossCurrencyInstrumentDescription, optional

    """

    def __init__(
        self,
        cross_currency_swaps: Optional[List[CrossCurrencyInstrumentDescription]] = None,
        fx_forwards: Optional[List[FxForwardInstrumentDescription]] = None,
        fx_spot: Optional[FxSpotInstrumentDescription] = None,
        interest_rate_swaps: Optional[List[CrossCurrencyInstrumentDescription]] = None,
        overnight_index_swaps: Optional[
            List[CrossCurrencyInstrumentDescription]
        ] = None,
    ) -> None:
        super().__init__()
        self.cross_currency_swaps = try_copy_to_list(cross_currency_swaps)
        self.fx_forwards = try_copy_to_list(fx_forwards)
        self.fx_spot = fx_spot
        self.interest_rate_swaps = try_copy_to_list(interest_rate_swaps)
        self.overnight_index_swaps = try_copy_to_list(overnight_index_swaps)

    @property
    def cross_currency_swaps(self):
        """
        :return: list CrossCurrencyInstrumentDescription
        """
        return self._get_list_parameter(
            CrossCurrencyInstrumentDescription, "crossCurrencySwaps"
        )

    @cross_currency_swaps.setter
    def cross_currency_swaps(self, value):
        self._set_list_parameter(
            CrossCurrencyInstrumentDescription, "crossCurrencySwaps", value
        )

    @property
    def fx_forwards(self):
        """
        :return: list FxForwardInstrumentDescription
        """
        return self._get_list_parameter(FxForwardInstrumentDescription, "fxForwards")

    @fx_forwards.setter
    def fx_forwards(self, value):
        self._set_list_parameter(FxForwardInstrumentDescription, "fxForwards", value)

    @property
    def fx_spot(self):
        """
        :return: object FxSpotInstrumentDescription
        """
        return self._get_object_parameter(FxSpotInstrumentDescription, "fxSpot")

    @fx_spot.setter
    def fx_spot(self, value):
        self._set_object_parameter(FxSpotInstrumentDescription, "fxSpot", value)

    @property
    def interest_rate_swaps(self):
        """
        :return: list CrossCurrencyInstrumentDescription
        """
        return self._get_list_parameter(
            CrossCurrencyInstrumentDescription, "interestRateSwaps"
        )

    @interest_rate_swaps.setter
    def interest_rate_swaps(self, value):
        self._set_list_parameter(
            CrossCurrencyInstrumentDescription, "interestRateSwaps", value
        )

    @property
    def overnight_index_swaps(self):
        """
        :return: list CrossCurrencyInstrumentDescription
        """
        return self._get_list_parameter(
            CrossCurrencyInstrumentDescription, "overnightIndexSwaps"
        )

    @overnight_index_swaps.setter
    def overnight_index_swaps(self, value):
        self._set_list_parameter(
            CrossCurrencyInstrumentDescription, "overnightIndexSwaps", value
        )



# coding: utf8

from typing import Optional

from .._instrument_definition import ObjectDefinition
from ._enums import (
    Direction,
    DocClause,
    Seniority,
)


class ProtectionLegDefinition(ObjectDefinition):
    """
    API endpoint for Financial Contract analytics,
    that returns calculations relevant to each contract type.

    Parameters
    ----------
    direction : Direction, optional
        The direction of the leg. the possible values are:
        - 'Paid' (the cash flows of the leg are paid to the counterparty),
        - 'Received' (the cash flows of the leg are received from the counterparty).
          Optional for a single leg instrument (like a bond), in that case default value
          is Received. It is mandatory for a multi-instrument leg instrument (like Swap
          or CDS leg).
    notional_ccy : str, optional
        The ISO code of the notional currency. Mandatory if instrument code or
        instrument style has not been defined. In case an instrument code/style has been
        defined, value may comes from the reference data.
    notional_amount : float, optional
        The notional amount of the leg at the period start date. Optional. By default
        1,000,000 is used.
    doc_clause : DocClause, optional
        The restructuring clause or credit event for Single Name Cds. The possible
        values are:
        - CumRestruct14,
        - ModifiedRestruct14,
        - ModModRestruct14,
        - ExRestruct14,
        - CumRestruct03,
        - ModifiedRestruct03,
        - ModModRestruct03,
        - ExRestruct03. Optional. By default the doc_clause of the reference_entity's
          Primary Ric is used.
    seniority : Seniority, optional
        The order of repayment in the case of a credit event for Single Name Cds. The
        possible values are:
        - Secured (Secured Debt (Corporate/Financial) or Domestic Currency Sovereign
          Debt (Government)),
        - SeniorUnsecured (Senior Unsecured Debt (Corporate/Financial) or Foreign
          Currency Sovereign Debt (Government)),
        - Subordinated (Subordinated or Lower Tier 2 Debt (Banks)),
        - JuniorSubordinated (Junior Subordinated or Upper Tier 2 Debt (Banks)),
        - Preference (Preference Shares or Tier 1 Capital (Banks)). Optional. By default
          the seniority of the reference_entity's Primary Ric is used.
    index_factor : float, optional
        The factor that is applied to the notional in case a credit event happens in one
        of the constituents of the Cds Index. Optional. By default no factor (1)
        applies.
    index_series : int, optional
        The series of the Cds Index.  Optional. By default the series of the BenchmarkRic
        is used.
    recovery_rate : float, optional
        The percentage of recovery in case of a credit event. Optional. By default the
        recovery_rate of the Cds built from reference_entity, seniority, doc_clause and
        notional_currency is used.
    recovery_rate_percent : float, optional
        The percentage of recovery in case of a credit event. Optional. By default the
        recovery_rate of the Cds built from reference_entity, seniority, doc_clause and
        notional_currency is used.
    reference_entity : str, optional
        The identifier of the reference entity, it can be:
        - for Single Name : a RedCode, an OrgId, a reference entity's RIC,
        - for Index : a RedCode, a ShortName, a CommonName. Mandatory.
    settlement_convention : str, optional
        The cashSettlementRule of the CDS. Optional. By default "3WD" (3 week days) is
        used.
    """

    def __init__(
        self,
        *,
        direction: Optional[Direction] = None,
        notional_ccy: Optional[str] = None,
        notional_amount: Optional[float] = None,
        doc_clause: Optional[DocClause] = None,
        seniority: Optional[Seniority] = None,
        index_factor: Optional[float] = None,
        index_series: Optional[int] = None,
        recovery_rate: Optional[float] = None,
        recovery_rate_percent: Optional[float] = None,
        reference_entity: Optional[str] = None,
        settlement_convention: Optional[str] = None,
    ) -> None:
        super().__init__()
        self.direction = direction
        self.notional_ccy = notional_ccy
        self.notional_amount = notional_amount
        self.doc_clause = doc_clause
        self.seniority = seniority
        self.index_factor = index_factor
        self.index_series = index_series
        self.recovery_rate = recovery_rate
        self.recovery_rate_percent = recovery_rate_percent
        self.reference_entity = reference_entity
        self.settlement_convention = settlement_convention

    @property
    def direction(self):
        """
        The direction of the leg. the possible values are:
        - 'Paid' (the cash flows of the leg are paid to the counterparty),
        - 'Received' (the cash flows of the leg are received from the counterparty).
          Optional for a single leg instrument (like a bond), in that case default value
          is Received. It is mandatory for a multi-instrument leg instrument (like Swap
          or CDS leg).
        :return: enum Direction
        """
        return self._get_enum_parameter(Direction, "direction")

    @direction.setter
    def direction(self, value):
        self._set_enum_parameter(Direction, "direction", value)

    @property
    def doc_clause(self):
        """
        The restructuring clause or credit event for Single Name Cds. The possible
        values are:
        - CumRestruct14,
        - ModifiedRestruct14,
        - ModModRestruct14,
        - ExRestruct14,
        - CumRestruct03,
        - ModifiedRestruct03,
        - ModModRestruct03,
        - ExRestruct03. Optional.
        By default the doc_clause of the reference_entity's Primary Ric is used.
        :return: enum DocClause
        """
        return self._get_enum_parameter(DocClause, "docClause")

    @doc_clause.setter
    def doc_clause(self, value):
        self._set_enum_parameter(DocClause, "docClause", value)

    @property
    def seniority(self):
        """
        The order of repayment in the case of a credit event for Single Name Cds. The
        possible values are:
        - Secured (Secured Debt (Corporate/Financial) or Domestic Currency Sovereign
          Debt (Government)),
        - SeniorUnsecured (Senior Unsecured Debt (Corporate/Financial) or Foreign
          Currency Sovereign Debt (Government)),
        - Subordinated (Subordinated or Lower Tier 2 Debt (Banks)),
        - JuniorSubordinated (Junior Subordinated or Upper Tier 2 Debt (Banks)),
        - Preference (Preference Shares or Tier 1 Capital (Banks)).
        Optional. By default the seniority of the reference_entity's
        Primary Ric is used.
        :return: enum Seniority
        """
        return self._get_enum_parameter(Seniority, "seniority")

    @seniority.setter
    def seniority(self, value):
        self._set_enum_parameter(Seniority, "seniority", value)

    @property
    def index_factor(self):
        """
        The factor that is applied to the notional in case a credit event happens in one
        of the constituents of the Cds Index. Optional. By default no factor (1)
        applies.
        :return: float
        """
        return self._get_parameter("indexFactor")

    @index_factor.setter
    def index_factor(self, value):
        self._set_parameter("indexFactor", value)

    @property
    def index_series(self):
        """
        The series of the Cds Index.  Optional.
        By default the series of the BenchmarkRic is used.
        :return: int
        """
        return self._get_parameter("indexSeries")

    @index_series.setter
    def index_series(self, value):
        self._set_parameter("indexSeries", value)

    @property
    def notional_amount(self):
        """
        The notional amount of the leg at the period start date. Optional. By default
        1,000,000 is used.
        :return: float
        """
        return self._get_parameter("notionalAmount")

    @notional_amount.setter
    def notional_amount(self, value):
        self._set_parameter("notionalAmount", value)

    @property
    def notional_ccy(self):
        """
        The ISO code of the notional currency. Mandatory if instrument code or
        instrument style has not been defined. In case an instrument code/style has been
        defined, value may comes from the reference data.
        :return: str
        """
        return self._get_parameter("notionalCcy")

    @notional_ccy.setter
    def notional_ccy(self, value):
        self._set_parameter("notionalCcy", value)

    @property
    def recovery_rate(self):
        """
        The percentage of recovery in case of a credit event. Optional. By default the
        recovery_rate of the Cds built from reference_entity, seniority, doc_clause and
        notional_currency is used.
        :return: float
        """
        return self._get_parameter("recoveryRate")

    @recovery_rate.setter
    def recovery_rate(self, value):
        self._set_parameter("recoveryRate", value)

    @property
    def recovery_rate_percent(self):
        """
        The percentage of recovery in case of a credit event. Optional. By default the
        recovery_rate of the Cds built from reference_entity, seniority, doc_clause and
        notional_currency is used.
        :return: float
        """
        return self._get_parameter("recoveryRatePercent")

    @recovery_rate_percent.setter
    def recovery_rate_percent(self, value):
        self._set_parameter("recoveryRatePercent", value)

    @property
    def reference_entity(self):
        """
        The identifier of the reference entity, it can be:
        - for Single Name : a RedCode, an OrgId, a reference entity's RIC,
        - for Index : a RedCode, a ShortName, a CommonName. Mandatory.
        :return: str
        """
        return self._get_parameter("referenceEntity")

    @reference_entity.setter
    def reference_entity(self, value):
        self._set_parameter("referenceEntity", value)

    @property
    def settlement_convention(self):
        """
        The cashSettlementRule of the CDS. Optional. By default "3WD" (3 week days) is
        used.
        :return: str
        """
        return self._get_parameter("settlementConvention")

    @settlement_convention.setter
    def settlement_convention(self, value):
        self._set_parameter("settlementConvention", value)



# coding: utf8

from typing import Optional
from ._enums import (
    IndexConvexityAdjustmentIntegrationMethod,
    IndexConvexityAdjustmentMethod,
    PriceSide,
)
from ..._object_definition import ObjectDefinition


class PricingParameters(ObjectDefinition):
    """
    API endpoint for Financial Contract analytics,
    that returns calculations relevant to each contract type.

    Parameters
    ----------
    index_convexity_adjustment_integration_method : IndexConvexityAdjustmentIntegrationMethod, optional

    index_convexity_adjustment_method : IndexConvexityAdjustmentMethod, optional

    price_side : PriceSide, optional
        The quoted price side of the instrument. optional. default value is 'mid'.
    market_data_date : str, optional
        The market data date for pricing. Optional. By default, the marketDataDate date
        is the ValuationDate or Today
    market_value_in_deal_ccy : float, optional
        MarketValueInDealCcy to override and that will be used as pricing analysis input
        to compute VolatilityPercent. Optional. No override is applied by default. Note
        that Premium takes priority over Volatility input.
    report_ccy : str, optional
        Valuation is performed in deal currency. If a report currency is set, valuation
        is done in that report currency.
    skip_first_cap_floorlet : bool, optional
        Indicates whether to take in consideration the first caplet
    valuation_date : str, optional
        The valuation date for pricing.  Optional. If not set the valuation date is
        equal to MarketDataDate or Today. For assets that contains a
        settlementConvention, the default valuation date  is equal to the settlementdate
        of the Asset that is usually the TradeDate+SettlementConvention.
    """

    def __init__(
        self,
        index_convexity_adjustment_integration_method: Optional[
            IndexConvexityAdjustmentIntegrationMethod
        ] = None,
        index_convexity_adjustment_method: Optional[
            IndexConvexityAdjustmentMethod
        ] = None,
        price_side: Optional[PriceSide] = None,
        market_data_date: Optional[str] = None,
        market_value_in_deal_ccy: Optional[float] = None,
        report_ccy: Optional[str] = None,
        skip_first_cap_floorlet: Optional[bool] = None,
        valuation_date: Optional[str] = None,
    ) -> None:
        super().__init__()
        self.index_convexity_adjustment_integration_method = (
            index_convexity_adjustment_integration_method
        )
        self.index_convexity_adjustment_method = index_convexity_adjustment_method
        self.price_side = price_side
        self.market_data_date = market_data_date
        self.market_value_in_deal_ccy = market_value_in_deal_ccy
        self.report_ccy = report_ccy
        self.skip_first_cap_floorlet = skip_first_cap_floorlet
        self.valuation_date = valuation_date

    @property
    def index_convexity_adjustment_integration_method(self):
        """
        :return: enum IndexConvexityAdjustmentIntegrationMethod
        """
        return self._get_enum_parameter(
            IndexConvexityAdjustmentIntegrationMethod,
            "indexConvexityAdjustmentIntegrationMethod",
        )

    @index_convexity_adjustment_integration_method.setter
    def index_convexity_adjustment_integration_method(self, value):
        self._set_enum_parameter(
            IndexConvexityAdjustmentIntegrationMethod,
            "indexConvexityAdjustmentIntegrationMethod",
            value,
        )

    @property
    def index_convexity_adjustment_method(self):
        """
        :return: enum IndexConvexityAdjustmentMethod
        """
        return self._get_enum_parameter(
            IndexConvexityAdjustmentMethod, "indexConvexityAdjustmentMethod"
        )

    @index_convexity_adjustment_method.setter
    def index_convexity_adjustment_method(self, value):
        self._set_enum_parameter(
            IndexConvexityAdjustmentMethod, "indexConvexityAdjustmentMethod", value
        )

    @property
    def price_side(self):
        """
        The quoted price side of the instrument. Optional. Default value is 'mid'.
        :return: enum PriceSide
        """
        return self._get_enum_parameter(PriceSide, "priceSide")

    @price_side.setter
    def price_side(self, value):
        self._set_enum_parameter(PriceSide, "priceSide", value)

    @property
    def market_data_date(self):
        """
        The market data date for pricing. Optional. By default, the marketDataDate date
        is the ValuationDate or Today
        :return: str
        """
        return self._get_parameter("marketDataDate")

    @market_data_date.setter
    def market_data_date(self, value):
        self._set_parameter("marketDataDate", value)

    @property
    def market_value_in_deal_ccy(self):
        """
        MarketValueInDealCcy to override and that will be used as pricing analysis input
        to compute VolatilityPercent. Optional. No override is applied by default. Note
        that Premium takes priority over Volatility input.
        :return: float
        """
        return self._get_parameter("marketValueInDealCcy")

    @market_value_in_deal_ccy.setter
    def market_value_in_deal_ccy(self, value):
        self._set_parameter("marketValueInDealCcy", value)

    @property
    def report_ccy(self):
        """
        Valuation is performed in deal currency. If a report currency is set, valuation
        is done in that report currency.
        :return: str
        """
        return self._get_parameter("reportCcy")

    @report_ccy.setter
    def report_ccy(self, value):
        self._set_parameter("reportCcy", value)

    @property
    def skip_first_cap_floorlet(self):
        """
        Indicates whether to take in consideration the first caplet
        :return: bool
        """
        return self._get_parameter("skipFirstCapFloorlet")

    @skip_first_cap_floorlet.setter
    def skip_first_cap_floorlet(self, value):
        self._set_parameter("skipFirstCapFloorlet", value)

    @property
    def valuation_date(self):
        """
        The valuation date for pricing.  Optional. If not set the valuation date is
        equal to MarketDataDate or Today. For assets that contains a
        settlementConvention, the default valuation date  is equal to the settlementdate
        of the Asset that is usually the TradeDate+SettlementConvention.
        :return: str
        """
        return self._get_parameter("valuationDate")

    @valuation_date.setter
    def valuation_date(self, value):
        self._set_parameter("valuationDate", value)



# coding: utf8

from typing import Optional, Union, TYPE_CHECKING

from ._enums import BuySell, CallPut, ExerciseStyle, UnderlyingType, SettlementType
from ._eti import (
    EtiUnderlyingDefinition,
    EtiBinaryDefinition,
    EtiBarrierDefinition,
    EtiCbbcDefinition,
    EtiDoubleBarriersDefinition,
    EtiFixingInfo,
)
from ._fx import (
    FxUnderlyingDefinition,
    FxBinaryDefinition,
    FxBarrierDefinition,
    FxDualCurrencyDefinition,
    FxAverageInfo,
    FxDoubleBarrierDefinition,
    FxDoubleBinaryDefinition,
    FxForwardStart,
)
from ._option_instrument_definition import OptionInstrumentDefinition
from ._option_pricing_parameters import PricingParameters
from .._base_definition import BaseDefinition
from ..._models import InputFlow
from ....._tools import validate_types, try_copy_to_list

if TYPE_CHECKING:
    from ....._types import ExtendedParams, OptStrStrs
    from ...financial_contracts._stream_facade import Stream
    from ....._core.session import Session


class Definition(BaseDefinition):
    """
    Parameters
    ----------
    instrument_tag : str, optional
        User defined string to identify the instrument.It can be used to link output
        results to the instrument definition. Only alphabetic, numeric and '- _.#=@'
        characters are supported. Optional.
    end_date : str, optional
        Expiry date of the option
    buy_sell : BuySell, optional
        The side of the deal. Possible values:
        - Buy
        - Sell
    call_put : CallPut, optional
        Tells if the option is a call or a put. Possible values:
        - Call
        - Put
    exercise_style : ExerciseStyle, optional
        EURO or AMER
    underlying_type : UnderlyingType, optional
        Underlying type of the option. Possible values:
        - Eti
        - Fx
    strike : float, optional
        strike of the option
    tenor : str, optional
        tenor of the option
    notional_ccy : str, optional
        Currency of the notional amount If the option is a EURGBP Call option,
        notional_ccy can be expressed in EUR OR GBP
    notional_amount : float, optional
        The notional amount of currency If the option is a EURGBP Call option, amount of
        EUR or GBP of the contract
    asian_definition : FxOptionAverageInfo, EtiOptionFixingInfo, optional
        Fixing details for asian options
    barrier_definition : FxOptionBarrierDefinition, EtiOptionBarrierDefinition, optional
        Details for barrier option.
    binary_definition : FxOptionBinaryDefinition, EtiOptionBinaryDefinition, optional
        Details for binary option.
    double_barrier_definition : FxOptionDoubleBarrierDefinition, optional
        Details for double barriers option.
    double_binary_definition : FxOptionDoubleBinaryDefinition, optional
        Details for double binary option.
    dual_currency_definition : FxDualCurrencyDefinition, optional
        Details for dual currency option.
    forward_start_definition : FxOptionForwardStart, optional
        Details for Forward Start option.
    underlying_definition : FxUnderlyingDefinition, EtiUnderlyingDefinition, optional
        Details of the underlying. Can be used to override some data of the underlying.
    delivery_date : str, optional
        Expiry date of the option
    instrument_code : str, optional
        An option RIC that is used to retrieve the description of the
        EtiOptionDefinition contract. Optional.If null, the instrument_code of
        underlying_definition must be provided.
    cbbc_definition : EtiOptionCbbcDefinition, optional
        Details for CBBC (Call Bear/Bull Contract) option.
    double_barriers_definition : EtiOptionDoubleBarriersDefinition, optional
        Details for double barriers option.
    deal_contract : int, optional
        deal_contract. It is the number of contracts bought or sold in the deal.
    end_date_time : str, optional
        Expiry date time of the option
    lot_size : float, optional
        The lot size. It is the number of options bought or sold in one transaction.
    offset : int, optional
        offset. The offset in minutes between the time UTC and the time of the exchange
        where the contract is traded.
    fields: list of str, optional
        Contains the list of Analytics that the quantitative analytic service will
        compute.
    pricing_parameters : PricingParameters, optional
        The pricing parameters to apply to this instrument. Optional. If pricing
        parameters are not provided at this level parameters defined globally at the
        request level are used. If no pricing parameters are provided globally default
        values apply.
    extended_params : dict, optional
        If necessary other parameters

    Methods
    -------
    get_data(session=session, on_response=on_response)
        Returns a response to the data platform
    get_stream(session=session)
        Get stream quantitative analytic service subscription

    Examples
    --------
     >>> import refinitiv.data.content.ipa.financial_contracts as rdf
     >>> definition = rdf.option.Definition(
     ...    instrument_code="FCHI560000L1.p",
     ...    underlying_type=rdf.option.UnderlyingType.ETI,
     ...    fields=[
     ...        "MarketValueInDealCcy",
     ...        "DeltaPercent",
     ...        "GammaPercent",
     ...        "RhoPercent",
     ...        "ThetaPercent",
     ...        "VegaPercent",
     ...        "ErrorCode",
     ...        "ErrorMessage",
     ...    ],
     ... )
     >>> response = definition.get_data()

     Using get_stream
     >>> response = definition.get_stream()
    """

    def __init__(
        self,
        asian_definition: Union[EtiFixingInfo, FxAverageInfo] = None,
        barrier_definition: Union[FxBarrierDefinition, EtiBarrierDefinition] = None,
        binary_definition: Union[FxBinaryDefinition, EtiBinaryDefinition] = None,
        buy_sell: Optional[BuySell] = None,
        call_put: Optional[CallPut] = None,
        cbbc_definition: Optional[EtiCbbcDefinition] = None,
        deal_contract: Optional[int] = None,
        delivery_date: Optional[str] = None,
        double_barrier_definition: Optional[FxDoubleBarrierDefinition] = None,
        double_barriers_definition: Optional[EtiDoubleBarriersDefinition] = None,
        double_binary_definition: Union[FxDoubleBinaryDefinition] = None,
        dual_currency_definition: Optional[FxDualCurrencyDefinition] = None,
        end_date: Optional[str] = None,
        end_date_time: Optional[str] = None,
        exercise_style: Optional[ExerciseStyle] = None,
        forward_start_definition: Optional[FxForwardStart] = None,
        instrument_code: Optional[str] = None,
        instrument_tag: Optional[str] = None,
        lot_size: Optional[float] = None,
        notional_amount: Optional[float] = None,
        notional_ccy: Optional[str] = None,
        payments: Optional[InputFlow] = None,
        settlement_ccy: Optional[str] = None,
        settlement_type: Optional[SettlementType] = None,
        start_date: Optional[str] = None,
        strike: Optional[float] = None,
        tenor: Optional[str] = None,
        time_zone_offset: Optional[int] = None,
        underlying_definition: Union[
            FxUnderlyingDefinition, EtiUnderlyingDefinition
        ] = None,
        underlying_type: Optional[UnderlyingType] = None,
        fields: "OptStrStrs" = None,
        pricing_parameters: Optional[PricingParameters] = None,
        extended_params: "ExtendedParams" = None,
    ):
        validate_types(deal_contract, [int, type(None)], "deal_contract")
        fields = try_copy_to_list(fields)

        definition = OptionInstrumentDefinition(
            asian_definition=asian_definition,
            barrier_definition=barrier_definition,
            binary_definition=binary_definition,
            buy_sell=buy_sell,
            call_put=call_put,
            cbbc_definition=cbbc_definition,
            deal_contract=deal_contract,
            delivery_date=delivery_date,
            double_barrier_definition=double_barrier_definition,
            double_barriers_definition=double_barriers_definition,
            double_binary_definition=double_binary_definition,
            dual_currency_definition=dual_currency_definition,
            end_date=end_date,
            end_date_time=end_date_time,
            exercise_style=exercise_style,
            forward_start_definition=forward_start_definition,
            instrument_code=instrument_code,
            instrument_tag=instrument_tag,
            lot_size=lot_size,
            notional_amount=notional_amount,
            notional_ccy=notional_ccy,
            payments=payments,
            settlement_ccy=settlement_ccy,
            settlement_type=settlement_type,
            start_date=start_date,
            strike=strike,
            tenor=tenor,
            time_zone_offset=time_zone_offset,
            underlying_definition=underlying_definition,
            underlying_type=underlying_type,
        )
        super().__init__(
            definition=definition,
            fields=fields,
            pricing_parameters=pricing_parameters,
            extended_params=extended_params,
        )

    def get_stream(self, session: Optional["Session"] = None) -> "Stream":
        fields = self._kwargs.get("fields")
        if fields is None:
            response = self.get_data(session=session)
            if isinstance(response.data.raw, dict) and "headers" in response.data.raw:
                fields = [item.get("name", "") for item in response.data.raw["headers"]]
                self._kwargs["fields"] = fields

        return super().get_stream(session=session)
